@@ -233,24 +233,13 @@ class FeedManager(gobject.GObject):
                 # new_entries to the old ones while updating the rest
                 # of the information
                 old_entries = feed_data.entries
-                feed_data.update(updated_feed_data)
+                # Do not overwrite visits, and sync values
+                feed_data.update(updated_feed_data, visits=feed_data['visits'], sync=feed_data['sync'])
                 feed_data.entries = old_entries + new_entries
 
-        # Save to disk
-        try:
-            self.save(None)
-        except IOError:
-            pass
-
     def remove_feed(self, feed):
         self.feed_data_list.remove(feed)
 
-        # Save to disk
-        try:
-            self.save(None)
-        except IOError:
-            pass
-
     @requireWorker
     def export_opml(self, file_path, callback, data=None):
         # Create a new worker to save the  file
@@ -630,7 +619,7 @@ class FeedManager(gobject.GObject):
             # Get items from feed excluding those with read status. It
             # indeed really sucks that we can only ask for the unread
             # items from *ALL* of our subscriptions
-            unread_url = constants.URL_USER + constants.STATE_SUFFIX + 'reading-list' + '?' + 'xt=user/-/' + constants.STATE_SUFFIX + 'read'
+            unread_url = constants.URL_USER + constants.STATE_SUFFIX + 'reading-list' + '?' + 'xt=user/-/' + constants.STATE_SUFFIX + 'read' + '&n=1000'
             unread_request = urllib2.Request(unread_url, None, auth_headers)
             unread_response = urllib2.urlopen(unread_request)
             unread_data = unread_response.read()
@@ -652,6 +641,7 @@ class FeedManager(gobject.GObject):
                     unread_dict[feed_source] = [ entry_link ]
                 else:
                     unread_dict[feed_source].append(entry_link)
+
             try:
                 nodes = doc.getElementsByTagName('gr:continuation')
                 if nodes==[]:
