@@ -88,6 +88,36 @@ def get_author_from_item(item):
 
     return author
 
+def get_feed_title_markup(feed_data):
+    return '<span foreground="%s">%s</span>' % \
+        (DEFAULT_TEXT_COLOR, glib.markup_escape_text(unescape(feed_data.feed.title)))
+
+def get_feed_subtitle_markup(feed_data):
+    if 'subtitle' in feed_data.feed:
+        secondary = feed_data.feed.subtitle
+    else:
+        if 'link' in  feed_data.feed:
+            secondary = feed_data.feed.link
+        else:
+            secondary = feed_data.href
+
+    return '<span foreground="%s">%s</span>' % \
+        (SECONDARY_TEXT_COLOR, glib.markup_escape_text(unescape(secondary)))
+
+def get_feed_icon_uri(feed_data):
+    if 'icon' in feed_data.feed:
+        return feed_data.feed.icon
+    elif 'link' in feed_data.feed:
+        return feed_data.feed.link
+    else:
+        return feed_data.href
+
+def get_visual_unread_text(unread_count):
+    if unread_count:
+        return '<span size="small"> %d </span>' % unread_count
+    else:
+        return ''
+
 class FeedsView(hildon.GtkTreeView):
 
     DUMMY_FEED_STATUS = -1
@@ -122,22 +152,6 @@ class FeedsView(hildon.GtkTreeView):
 
         # TODO: Fixed size to improve panning ?
 
-    def get_feed_title_markup(self, feed_data):
-        return '<span foreground="%s">%s</span>' % \
-            (DEFAULT_TEXT_COLOR, glib.markup_escape_text(unescape(feed_data.feed.title)))
-
-    def get_feed_subtitle_markup(self, feed_data):
-        if 'subtitle' in feed_data.feed:
-            secondary = feed_data.feed.subtitle
-        else:
-            if 'link' in  feed_data.feed:
-                secondary = feed_data.feed.link
-            else:
-                secondary = feed_data.href
-
-        return '<span foreground="%s">%s</span>' % \
-            (SECONDARY_TEXT_COLOR, glib.markup_escape_text(unescape(secondary)))
-
 class SettingsDialog(gtk.Dialog):
 
     def __init__(self, parent, settings):
@@ -437,8 +451,8 @@ class FeedsWindow(hildon.StackableWindow):
         dummy_feed_data.feed.title = _('Retrieving info...')
         store.set(feed_iter,
                   self.FEEDS_MODEL_PIXBUF_COLUMN, None,
-                  self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(dummy_feed_data),
-                  self.FEEDS_MODEL_SUBTITLE_COLUMN, self.view.get_feed_subtitle_markup(dummy_feed_data),
+                  self.FEEDS_MODEL_TITLE_COLUMN, get_feed_title_markup(dummy_feed_data),
+                  self.FEEDS_MODEL_SUBTITLE_COLUMN, get_feed_subtitle_markup(dummy_feed_data),
                   self.FEEDS_MODEL_DATA_COLUMN, dummy_feed_data,
                   self.FEEDS_MODEL_READ_COLUMN, '',
                   self.FEEDS_MODEL_VISITS_COLUMN, 0,
@@ -477,7 +491,7 @@ class FeedsWindow(hildon.StackableWindow):
         # Reset unread cache (the value have most likely changed).
         unread_count = len([entry for entry in feed_data.entries if entry.read == False])
         self.view.get_model().set(feed_iter, self.FEEDS_MODEL_READ_COLUMN,
-                                  self._get_visual_unread_text(unread_count))
+                                  get_visual_unread_text(unread_count))
 
         # Update sync status (could have changed)
         self.view.get_model().set(feed_iter, self.FEEDS_MODEL_SYNC_COLUMN, feed_data.sync)
@@ -681,12 +695,6 @@ class FeedsWindow(hildon.StackableWindow):
         # Ask manager to update feed
         self.manager.update_all(self._all_feeds_updated_cb, None)
 
-    def _get_visual_unread_text(self, unread_count):
-        if unread_count:
-            return '<span size="small"> %d </span>' % unread_count
-        else:
-            return ''
-
     def _all_feeds_updated_cb(self, retval, data, error):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
 
@@ -694,7 +702,7 @@ class FeedsWindow(hildon.StackableWindow):
         store = self.view.get_model()
         for row in store:
             unread_count = len([entry for entry in row[self.FEEDS_MODEL_DATA_COLUMN].entries if entry.read == False])
-            row[self.FEEDS_MODEL_READ_COLUMN] = self._get_visual_unread_text(unread_count)
+            row[self.FEEDS_MODEL_READ_COLUMN] = get_visual_unread_text(unread_count)
 
         # Do not update read/unread status if all the feeds failed to sync
         synced_feeds = [feed for feed in self.manager.get_feed_list() if feed.sync]
@@ -755,10 +763,10 @@ class FeedsWindow(hildon.StackableWindow):
         # Update model
         store.set(feed_iter,
                   self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf,
-                  self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(new_feed_data),
-                  self.FEEDS_MODEL_SUBTITLE_COLUMN, self.view.get_feed_subtitle_markup(new_feed_data),
+                  self.FEEDS_MODEL_TITLE_COLUMN, get_feed_title_markup(new_feed_data),
+                  self.FEEDS_MODEL_SUBTITLE_COLUMN, get_feed_subtitle_markup(new_feed_data),
                   self.FEEDS_MODEL_DATA_COLUMN, new_feed_data,
-                  self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(len(new_feed_data.entries)),
+                  self.FEEDS_MODEL_READ_COLUMN, get_visual_unread_text(len(new_feed_data.entries)),
                   self.FEEDS_MODEL_VISITS_COLUMN, new_feed_data.visits,
                   self.FEEDS_MODEL_SYNC_COLUMN, new_feed_data.sync,
                   self.FEEDS_MODEL_ID_COLUMN, hash(new_feed_data.href))
@@ -802,7 +810,7 @@ class FeedsWindow(hildon.StackableWindow):
             feed_data = row[self.FEEDS_MODEL_DATA_COLUMN]
             if feed_data.sync:
                 unread_count = len([entry for entry in feed_data.entries if entry.read == False])
-                row[self.FEEDS_MODEL_READ_COLUMN] = self._get_visual_unread_text(unread_count)
+                row[self.FEEDS_MODEL_READ_COLUMN] = get_visual_unread_text(unread_count)
 
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
 
@@ -950,18 +958,17 @@ class FeedsWindow(hildon.StackableWindow):
                 feed_iter = store.append()
                 unread_count = len([entry for entry in feed_data.entries if entry.read == False])
                 store.set(feed_iter,
-                          self.FEEDS_MODEL_TITLE_COLUMN, self.view.get_feed_title_markup(feed_data),
-                          self.FEEDS_MODEL_SUBTITLE_COLUMN, self.view.get_feed_subtitle_markup(feed_data),
+                          self.FEEDS_MODEL_TITLE_COLUMN, get_feed_title_markup(feed_data),
+                          self.FEEDS_MODEL_SUBTITLE_COLUMN, get_feed_subtitle_markup(feed_data),
                           self.FEEDS_MODEL_DATA_COLUMN, feed_data,
-                          self.FEEDS_MODEL_READ_COLUMN, self._get_visual_unread_text(unread_count),
+                          self.FEEDS_MODEL_READ_COLUMN, get_visual_unread_text(unread_count),
                           self.FEEDS_MODEL_VISITS_COLUMN, feed_data.visits,
                           self.FEEDS_MODEL_SYNC_COLUMN, feed_data.sync,
                           self.FEEDS_MODEL_ID_COLUMN, hash(feed_data.href))
                 # Favicon might require a network download
                 path = self.view.get_model().get_path(feed_iter)
                 row_reference = gtk.TreeRowReference(store, path)
-                pixbuf = self.manager.get_favicon(feed_data.feed.link or feed_data.href,
-                                             self._get_favicon_cb, row_reference)
+                pixbuf = self.manager.get_favicon(get_feed_icon_uri(feed_data), self._get_favicon_cb, row_reference)
 
         # If there is no data then show a dialog
         if len(feed_data_list) == 0:
@@ -1234,9 +1241,16 @@ class EntriesWindow(hildon.StackableWindow):
         self.connect('configure-event', self._on_configuration_changed)
 
         # Force online/offline status evaluation
-        self._conn_manager.connect('connection-changed', self._on_connection_changed)
+        self._connection_changed_id = self._conn_manager.connect('connection-changed', self._on_connection_changed)
         self._on_connection_changed(self._conn_manager)
 
+    def __del__(self):
+        # Do not listen for conn-changed events anymore. Causes
+        # crashes when changing the connection after closing the
+        # window
+        if self._conn_manager.handler_is_connected(self._connection_changed_id):
+            self._conn_manager.disconnect(self._connection_changed_id)
+
     def _row_visible_cb(self, model, iter):
         if self.settings.entries_filter == constants.SHOW_ALL_FILTER:
             return True
@@ -1380,9 +1394,6 @@ class EntriesWindow(hildon.StackableWindow):
             hildon.hildon_banner_show_information(self, '', _('Error syncing read status with Google Reader'))
             return
 
-        self.view.get_model().get_model().clear()
-        self._add_entries()
-
     def _mark_read_cb(self, button, read):
         # tree view edit mode
         hildon.hildon_gtk_tree_view_set_ui_mode(self.view, gtk.HILDON_UI_MODE_EDIT)
@@ -1413,8 +1424,10 @@ class EntriesWindow(hildon.StackableWindow):
 
         online = self._conn_manager.is_online()
         for path in paths:
-            model[path][self.ENTRIES_MODEL_DATA_COLUMN].read = read
-            model[path][self.ENTRIES_MODEL_TEXT_COLUMN] = self.view.get_visual_entry_text(model[path][2])
+            entry = model[path][self.ENTRIES_MODEL_DATA_COLUMN]
+            entry.read = read
+            child_path = model.convert_path_to_child_path(path)
+            model.get_model()[child_path][self.ENTRIES_MODEL_TEXT_COLUMN] = self.view.get_visual_entry_text(entry)
             if self.feed_data.sync and online:
                 self.manager.mark_as_read_synchronize(self.feed_data.href, model[path][2].link,
                                                       read, self._mark_as_read_sync_cb)
@@ -1439,16 +1452,17 @@ class EntriesWindow(hildon.StackableWindow):
         item_window = ItemWindow(self.feed_data, self.manager, self.settings, self._conn_manager)
 
         item_window.connect('item-read', self._item_read_cb)
-        item_window.set_item_from_model(treeview.get_model().get_model(), path)
+        item_window.set_item_from_model(treeview.get_model(), path)
         item_window.show()
 
     def _item_read_cb(self, item_window, path):
-        entry = self.view.get_model()[path][self.ENTRIES_MODEL_DATA_COLUMN]
+        filter_model = self.view.get_model()
+        entry = filter_model[path][self.ENTRIES_MODEL_DATA_COLUMN]
 
         # Set item as read and update the UI
         entry['read'] = True
-        self.view.get_model().get_model()[path][self.ENTRIES_MODEL_TEXT_COLUMN] = \
-            self.view.get_visual_entry_text(entry)
+        child_path = filter_model.convert_path_to_child_path(path)
+        filter_model.get_model()[child_path][self.ENTRIES_MODEL_TEXT_COLUMN] = self.view.get_visual_entry_text(entry)
 
         # Mark item as read in Google Reader. Do it in an idle as we
         # don't want to delay the item rendering
@@ -1485,10 +1499,9 @@ class EntriesWindow(hildon.StackableWindow):
         else:
             hildon.hildon_gtk_window_set_progress_indicator(self, False)
             self._update_feed_button.set_sensitive(True)
-            # No need to update self.feed_data as it is automatically updated.
-            # The same happens with the reference stored in the FeedsWindow :-)
-            self.view.get_model().get_model().clear()
-            self._add_entries()
+
+        self.view.get_model().get_model().clear()
+        self._add_entries()
 
     def _download_all_items_cb(self, button):
         folder = self.settings.auto_download_folder
@@ -1587,6 +1600,9 @@ class ItemWindow(hildon.StackableWindow):
         self._conn_manager = conn_manager
         self._conn_manager.connect('connection-changed', self._on_connection_changed)
 
+        self._prev_item_reference = None
+        self._next_item_reference = None
+
         # Header
         self.item_window_header = ItemWindowHeader(False, 16)
         self.item_window_header.button_up.connect("clicked", self._up_button_clicked)
@@ -1748,43 +1764,52 @@ class ItemWindow(hildon.StackableWindow):
         dialog.show()
 
     def _up_button_clicked(self, button):
-
-        # http://faq.pygtk.org/index.py?req=show&file=faq13.051.htp
-        path = self.model.get_path(self.model_iter)
-        position = path[-1]
-        if position == 0:
-            return
-        prev_path = list(path)[:-1]
-        prev_path.append(position - 1)
-        self.model_iter = self.model.get_iter(tuple(prev_path))
-
-        if self.model_iter == None:
+        if not self._prev_item_reference or not self._prev_item_reference.valid():
             return
 
-        item = self.model.get_value(self.model_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
-        self.set_item(item)
+        prev_path = self._prev_item_reference.get_path()
+        prev_iter = self.model.get_iter(prev_path)
+        if prev_iter:
+            item = self.model.get_value(prev_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
+            self._set_item(item, prev_path)
 
     def _down_button_clicked(self, button):
-
-        if self.model_iter == None:
+        if not self._next_item_reference or not self._next_item_reference.valid():
             return
 
-        self.model_iter = self.model.iter_next(self.model_iter)
+        next_path = self._next_item_reference.get_path()
+        next_iter = self.model.get_iter(next_path)
+        if next_iter:
+            item = self.model.get_value(next_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
+            self._set_item(item, next_path)
 
-        item = self.model.get_value(self.model_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
-        self.set_item(item)
+    def _update_next_prev_references(self, path):
+        # Work out previous item path
+        # http://faq.pygtk.org/index.py?req=show&file=faq13.051.htp
+        position = path[-1]
+        if position:
+            prev_path = list(path)[:-1]
+            prev_path.append(position - 1)
+            self._prev_item_reference = gtk.TreeRowReference(self.model, tuple(prev_path))
+        else:
+            self._prev_item_reference = None
+
+        # Next item path
+        next_iter = self.model.iter_next(self.model.get_iter(path))
+        if next_iter:
+            self._next_item_reference = gtk.TreeRowReference(self.model, self.model.get_path(next_iter))
+        else:
+            self._next_item_reference = None
 
     def set_item_from_model(self, model, path):
-
         self.model = model
-        self.model_iter = model.get_iter(path)
+        item = model.get_value(model.get_iter(path), EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
+        self._set_item(item, path)
 
-        item_iter = model.get_iter(path)
-        item = model.get_value(item_iter, EntriesWindow.ENTRIES_MODEL_DATA_COLUMN)
+    def _set_item(self, item, path):
 
-        self.set_item(item)
-
-    def set_item(self, item):
+        # Update next&prev references
+        self._update_next_prev_references(path)
 
         author = get_author_from_item(item)
         if author == '':
@@ -1821,24 +1846,17 @@ class ItemWindow(hildon.StackableWindow):
 
         # Update button sensitiviness
         toolbar_button_list = self.toolbar.get_children()
-        path = self.model.get_path(self.model_iter)
-        position = path[-1]
-        if position == 0:
-            self.item_window_header.button_up.set_sensitive(False)
-            toolbar_button_list[0].set_sensitive(False)
-        elif position == len(self.model) - 1:
-            self.item_window_header.button_down.set_sensitive(False)
-            toolbar_button_list[1].set_sensitive(False)
-        else:
-            self.item_window_header.button_up.set_sensitive(True)
-            self.item_window_header.button_down.set_sensitive(True)
-            toolbar_button_list[0].set_sensitive(True)
-            toolbar_button_list[1].set_sensitive(True)
+        prev_item_missing = not self._prev_item_reference or not self._prev_item_reference.valid()
+        next_item_missing = not self._next_item_reference or not self._next_item_reference.valid()
+
+        self.item_window_header.button_up.set_sensitive(not prev_item_missing)
+        toolbar_button_list[0].set_sensitive(not prev_item_missing)
+        self.item_window_header.button_down.set_sensitive(not next_item_missing)
+        toolbar_button_list[1].set_sensitive(not next_item_missing)
 
         # Mark item as read
         if not item.read:
-            item.read = True
-            self.emit('item-read', self.model.get_path(self.model_iter))
+            self.emit('item-read', path)
 
         # Force online/offline status evaluation
         self._on_connection_changed(self._conn_manager)
